/*
 *    Copyright (C) 2006 Richard Kotal
 *
 *    This library is free software; you can redistribute it and/or modify it 
 *    under the terms of the GNU Library General Public License as published 
 *    by the Free Software Foundation; either version 2 of the License, or 
 *    (at your option) any later version.
 *
 *    This library is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *    Library General Public License for more details.
 *
 *    You should have received a copy of the GNU Library General Public
 *    License along with this library; if not, write to the
 *    Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 *    Boston, MA 02111-1307, USA.
 */


static int
ns_mhash_TclInterpInit (Tcl_Interp * interp, void *ctx)
{
  Tcl_CreateObjCommand (interp, TCLCMD, ns_mhash_TclCmdRegister, ctx, NULL);

  return NS_OK;
}


static int
ns_mhash_TclCmdRegister (ClientData arg, Tcl_Interp * interp, int objc, Tcl_Obj * CONST objv[])
{
  static CONST2 char *commands[] = { "keygen", "hash", "hmac", NULL };
  enum { KeygenIdx, HashIdx, HmacIdx } cmd;

  if (objc < 2)
    {
      Tcl_WrongNumArgs (interp, 1, objv, "command ? option arg ... ?");
      return TCL_ERROR;
    }
  if (Tcl_GetIndexFromObj (interp, objv[1], commands, "command", 0, (int *) &cmd) != TCL_OK)
    return TCL_ERROR;
  switch (cmd)
    {
    case KeygenIdx:
      {
	static CONST2 char *opts[] = { "algorithms", "create", "capability", NULL };
	enum { AlgoIdx, CreateIdx, CapaIdx } opt;
	if (Tcl_GetIndexFromObj
	    (interp, objv[2], opts, "command for keygen", 0,
	     (int *) &opt) != TCL_OK)
	  return TCL_ERROR;
	switch (opt)
	  {
	  case AlgoIdx:
	    {
	      return ns_mhash_KeygenAlgo (interp);
	      break;
	    }			/* AlgoIdx */
	  case CapaIdx:
	    {
	      char *algoname = NULL;
	      if (objc != 4)
		{
		  Tcl_WrongNumArgs (interp, 3, objv, "keygen_algorithm_name");
		  return TCL_ERROR;
		}
	      algoname = Tcl_GetString (objv[3]);
	      return ns_mhash_KeygenCapa (interp, algoname);
	      break;
	    }			/* CapaIdx */

	  case CreateIdx:
	    {
	      int bin = 0;
	      int idx = 3;
	      char *pass = NULL;
	      char *salt = NULL;
	      char *algoname = NULL;
	      int count = 0;
	      Tcl_Obj **hashv = NULL;
	      int hashc = 0;
	      int keysize = 0;
	      if (objc < 7 || objc > 10)
		{
		  Tcl_WrongNumArgs (interp, 3, objv, "?-bin|-base64? keygen_algorithm_name password list_of_hash_algorithms keysize ?repeated_hash_count? ?salt?");
		  return TCL_ERROR;
		}
	      if (STREQ (Tcl_GetString (objv[3]), "-bin"))
		{
		  bin = 1;
		  idx = 4;
		}
	      else if (STREQ (Tcl_GetString (objv[3]), "-base64"))
		{
		  bin = 2;
		  idx = 4;
		}
	      if (objc >= idx + 1)
		algoname = Tcl_GetString (objv[idx]);
	      idx++;
	      if (objc >= idx + 1)
		pass = Tcl_GetString (objv[idx]);
	      idx++;
	      if (objc >= idx + 1)
		{
		  int code = TCL_OK;
		  code = Tcl_ListObjGetElements (interp, objv[idx], &hashc, &hashv);
		  if (code != TCL_OK)
		    return TCL_ERROR;
		}
	      idx++;
	      if (objc >= idx + 1)
		Tcl_GetIntFromObj (interp, objv[idx], &keysize);
	      idx++;
	      if (objc >= idx + 1)
		Tcl_GetIntFromObj (interp, objv[idx], &count);
	      idx++;
	      if (objc >= idx + 1)
		salt = Tcl_GetString (objv[idx]);
	      return ns_mhash_KeygenCreate (interp, bin, algoname, pass, keysize, salt, count, hashc, hashv);
	      break;
	    }			/* CreateIdx */
	  }			/* opt - KeygenIdx */
	break;
      }				/* KeygenIdx */
    case HashIdx:
      {
	static CONST2 char *opts[] = { "algorithms", "new", "destroy", "update", "detach", "attach", "copy", "end", "serialize", "unserialize", NULL };
	enum { AlgoIdx, NewIdx, DestroyIdx, UpdateIdx, PutShareIdx, GetShareIdx, CopyIdx, EndIdx, SerializeIdx, UnserializeIdx } opt;
	if (Tcl_GetIndexFromObj (interp, objv[2], opts, "command for hash", 0, (int *) &opt) != TCL_OK)
	  return TCL_ERROR;
	switch (opt)
	  {
	  case AlgoIdx:
	    {
	      ns_mhash_HashAlgo (interp, STR_HASH);
	      break;
	    }			/* AlgoIdx */
	  case NewIdx:
	    {
	      char *algoname = NULL;
	      if (objc != 4)
		{
		  Tcl_WrongNumArgs (interp, 3, objv, "hash_algorithm_name");
		  return TCL_ERROR;
		}
	      algoname = Tcl_GetString (objv[3]);
	      return ns_mhash_HashNew (interp, algoname);
	      break;
	    }			/* NewIdx */
	  case DestroyIdx:
	    {
	      if (objc != 4)
		{
		  Tcl_WrongNumArgs (interp, 3, objv, "hashObjId");
		  return TCL_ERROR;
		}
	      return ns_mhash_DestroyHashObj (objv[3], STR_HASH);
	      break;
	    }			/* DestroyIdx */
	  case PutShareIdx:
	    {
	      if (objc != 4)
		{
		  Tcl_WrongNumArgs (interp, 3, objv, "hashObjId");
		  return TCL_ERROR;
		}
	      return ns_mhash_PutShareHashObj (interp, objv[3], STR_HASH);
	      break;
	    }			/* PutShareIdx */
	  case GetShareIdx:
	    {
	      if (objc != 4)
		{
		  Tcl_WrongNumArgs (interp, 3, objv, "hashDetachId");
		  return TCL_ERROR;
		}
	      return ns_mhash_GetShareHashObj (interp, objv[3], STR_HASH);
	      break;
	    }			/* GetShareIdx */
	  case SerializeIdx:
	    {
	      if (objc != 4)
		{
		  Tcl_WrongNumArgs (interp, 3, objv, "hashObjId");
		  return TCL_ERROR;
		}
	      return ns_mhash_SerializeHashObj (interp, objv[3], STR_HASH);
	      break;
	    }			/* SerializeIdx */
	  case UnserializeIdx:
	    {
	      if (objc != 4)
		{
		  Tcl_WrongNumArgs (interp, 3, objv, "serialize_string");
		  return TCL_ERROR;
		}
	      return ns_mhash_UnserializeHashObj (interp, objv[3], STR_HASH);
	      break;
	    }			/* UnserializeIdx */

	  case CopyIdx:
	    {
	      if (objc != 4)
		{
		  Tcl_WrongNumArgs (interp, 3, objv, "hashObjId");
		  return TCL_ERROR;
		}
	      return ns_mhash_CopyHashObj (interp, objv[3], STR_HASH);
	      break;
	    }			/* CopyIdx */
	  case EndIdx:
	    {
	      int idx = 3;
	      int bin = 0;
	      if (objc < 4 || objc > 5)
		{
		  Tcl_WrongNumArgs (interp, 3, objv, "?-bin|-base64? hashObjId");
		  return TCL_ERROR;
		}
	      if (STREQ (Tcl_GetString (objv[3]), "-bin"))
		{
		  bin = 1;
		  idx = 4;
		}
	      else if (STREQ (Tcl_GetString (objv[3]), "-base64"))
		{
		  bin = 2;
		  idx = 4;
		}
	      return ns_mhash_DigestPrintFromHash (interp, objv[idx], bin, STR_HASH);
	      break;
	    }			/* EndIdx */
	  case UpdateIdx:
	    {
	      char *txt = NULL;
	      int nbytes = 0;
	      if (objc != 5)
		{
		  Tcl_WrongNumArgs (interp, 3, objv, "hashObjId string");
		  return TCL_ERROR;
		}
	      txt = Tcl_GetByteArrayFromObj (objv[4], &nbytes);
	      return ns_mhash_UpdateHash (interp, objv[3], txt, nbytes, STR_HASH);
	      break;
	    }			/* UpdateIdx */

	  }
	break;
      }				/* HashIdx */
    case HmacIdx:
      {
	static CONST2 char *opts[] = { "algorithms", "new", "destroy", "update", "detach", "attach", "copy", "end", NULL };
	enum { AlgoIdx, NewIdx, DestroyIdx, UpdateIdx, PutShareIdx, GetShareIdx, CopyIdx, EndIdx } opt;
	if (Tcl_GetIndexFromObj (interp, objv[2], opts, "command for hmac", 0, (int *) &opt) != TCL_OK)
	  return TCL_ERROR;
	switch (opt)
	  {
	  case AlgoIdx:
	    {
	      ns_mhash_HashAlgo (interp, STR_HMAC);
	      break;
	    }			/* AlgoIdx */

	  case NewIdx:
	    {
	      char *algoname = NULL;
	      char *keyword = NULL;
	      int key_size = 0;
	      if (objc != 5)
		{
		  Tcl_WrongNumArgs (interp, 3, objv, "hmac_algorithm_name keyword");
		  return TCL_ERROR;
		}
	      algoname = Tcl_GetString (objv[3]);
	      keyword = Tcl_GetByteArrayFromObj (objv[4], &key_size);
	      return ns_mhash_HmacNew (interp, algoname, keyword, key_size);
	      break;
	    }			/* NewIdx */
	  case DestroyIdx:
	    {
	      if (objc != 4)
		{
		  Tcl_WrongNumArgs (interp, 3, objv, "hashObjId");
		  return TCL_ERROR;
		}
	      return ns_mhash_DestroyHashObj (objv[3], STR_HMAC);
	      break;
	    }			/* DestroyIdx */
	  case PutShareIdx:
	    {
	      if (objc != 4)
		{
		  Tcl_WrongNumArgs (interp, 3, objv, "hashObjId");
		  return TCL_ERROR;
		}
	      return ns_mhash_PutShareHashObj (interp, objv[3], STR_HMAC);
	      break;
	    }			/* PutShareIdx */
	  case GetShareIdx:
	    {
	      if (objc != 4)
		{
		  Tcl_WrongNumArgs (interp, 3, objv, "hashDetachId");
		  return TCL_ERROR;
		}
	      return ns_mhash_GetShareHashObj (interp, objv[3], STR_HMAC);
	      break;
	    }			/* GetShareIdx */
	  case CopyIdx:
	    {
	      if (objc != 4)
		{
		  Tcl_WrongNumArgs (interp, 3, objv, "hashObjId");
		  return TCL_ERROR;
		}
	      return ns_mhash_CopyHashObj (interp, objv[3], STR_HMAC);
	      break;
	    }			/* CopyIdx */
	  case EndIdx:
	    {
	      int idx = 3;
	      int bin = 0;
	      if (objc < 4 || objc > 5)
		{
		  Tcl_WrongNumArgs (interp, 3, objv, "?-bin|-base64? hashObjId");
		  return TCL_ERROR;
		}
	      if (STREQ (Tcl_GetString (objv[3]), "-bin"))
		{
		  bin = 1;
		  idx = 4;
		}
	      else if (STREQ (Tcl_GetString (objv[3]), "-base64"))
		{
		  bin = 2;
		  idx = 4;
		}
	      return ns_mhash_DigestPrintFromHash (interp, objv[idx], bin, STR_HMAC);
	      break;
	    }			/* EndIdx */

	  case UpdateIdx:
	    {
	      char *txt = NULL;
	      int nbytes = 0;
	      if (objc != 5)
		{
		  Tcl_WrongNumArgs (interp, 3, objv, "hashObjId string");
		  return TCL_ERROR;
		}
	      txt = Tcl_GetByteArrayFromObj (objv[4], &nbytes);
	      return ns_mhash_UpdateHash (interp, objv[3], txt, nbytes, STR_HMAC);
	      break;
	    }			/* UpdateIdx */

	  }
	break;
      }				/* HmacIdx */
    }				/* cmd */
  return TCL_OK;
}
